/* 
 * Octopus file import API
 *
 * Rest API to load documents in Octopus
 *
 * OpenAPI spec version: 1.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using IO.Swagger.Client;
using IO.Swagger.Model;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace IO.Swagger.Api
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public interface IDocumentsApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// Import document
        /// </summary>
        /// <remarks>
        /// Accept Octopus mailbox checksum and attachement
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns></returns>
        void LoadDocument(AttachementData body, int? checksum, int? dossierId, string subfolder = null);

        /// <summary>
        /// Import document
        /// </summary>
        /// <remarks>
        /// Accept Octopus mailbox checksum and attachement
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<Object> LoadDocumentWithHttpInfo(AttachementData body, int? checksum, int? dossierId, string subfolder = null);
        #endregion Synchronous Operations
        #region Asynchronous Operations
        /// <summary>
        /// Import document
        /// </summary>
        /// <remarks>
        /// Accept Octopus mailbox checksum and attachement
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task LoadDocumentAsync(AttachementData body, int? checksum, int? dossierId, string subfolder = null);

        /// <summary>
        /// Import document
        /// </summary>
        /// <remarks>
        /// Accept Octopus mailbox checksum and attachement
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<Object>> LoadDocumentAsyncWithHttpInfo(AttachementData body, int? checksum, int? dossierId, string subfolder = null);
        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public partial class DocumentsApi : IDocumentsApi
    {
        private IO.Swagger.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentsApi"/> class.
        /// </summary>
        /// <returns></returns>
        public DocumentsApi(String basePath)
        {
            this.Configuration = new IO.Swagger.Client.Configuration { BasePath = basePath };

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentsApi"/> class
        /// </summary>
        /// <returns></returns>
        public DocumentsApi()
        {
            this.Configuration = IO.Swagger.Client.Configuration.Default;

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentsApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public DocumentsApi(IO.Swagger.Client.Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = IO.Swagger.Client.Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public IO.Swagger.Client.Configuration Configuration { get; set; }

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public IO.Swagger.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public IDictionary<String, String> DefaultHeader()
        {
            return new ReadOnlyDictionary<string, string>(this.Configuration.DefaultHeader);
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        /// Import document Accept Octopus mailbox checksum and attachement
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns></returns>
        public void LoadDocument(AttachementData body, int? checksum, int? dossierId, string subfolder = null)
        {
            LoadDocumentWithHttpInfo(body, checksum, dossierId, subfolder);
        }

        /// <summary>
        /// Import document Accept Octopus mailbox checksum and attachement
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<Object> LoadDocumentWithHttpInfo(AttachementData body, int? checksum, int? dossierId, string subfolder = null)
        {
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling DocumentsApi->LoadDocument");
            // verify the required parameter 'checksum' is set
            if (checksum == null)
                throw new ApiException(400, "Missing required parameter 'checksum' when calling DocumentsApi->LoadDocument");
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling DocumentsApi->LoadDocument");

            var localVarPath = "/dossiers/{dossierId}/files/imports";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (subfolder != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "subfolder", subfolder)); // query parameter
            if (checksum != null) localVarHeaderParams.Add("checksum", this.Configuration.ApiClient.ParameterToString(checksum)); // header parameter
            if (body != null && body.GetType() != typeof(byte[]))
            {
                localVarPostBody = this.Configuration.ApiClient.Serialize(body); // http body (model) parameter
            }
            else
            {
                localVarPostBody = body; // byte array
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)this.Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("LoadDocument", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                null);
        }

        /// <summary>
        /// Import document Accept Octopus mailbox checksum and attachement
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task LoadDocumentAsync(AttachementData body, int? checksum, int? dossierId, string subfolder = null)
        {
            await LoadDocumentAsyncWithHttpInfo(body, checksum, dossierId, subfolder);

        }

        /// <summary>
        /// Import document Accept Octopus mailbox checksum and attachement
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Document to import</param>
        /// <param name="checksum">Octopus checksum</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="subfolder">Import location subfolder (optional)</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<Object>> LoadDocumentAsyncWithHttpInfo(AttachementData body, int? checksum, int? dossierId, string subfolder = null)
        {
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling DocumentsApi->LoadDocument");
            // verify the required parameter 'checksum' is set
            if (checksum == null)
                throw new ApiException(400, "Missing required parameter 'checksum' when calling DocumentsApi->LoadDocument");
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling DocumentsApi->LoadDocument");

            var localVarPath = "/dossiers/{dossierId}/files/imports";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (subfolder != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "subfolder", subfolder)); // query parameter
            if (checksum != null) localVarHeaderParams.Add("checksum", this.Configuration.ApiClient.ParameterToString(checksum)); // header parameter
            if (body != null && body.GetType() != typeof(byte[]))
            {
                localVarPostBody = this.Configuration.ApiClient.Serialize(body); // http body (model) parameter
            }
            else
            {
                localVarPostBody = body; // byte array
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("LoadDocument", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                null);
        }

    }
}
